#!build/env/bin/python
# Copyright (c) 2013 Cloudera, Inc. All rights reserved.

import logging
try:
  import json
except:
  import simplejson as json

__VERSION__ = '0.1'

LOG = logging.getLogger(__name__)

class SimpleModule(object):
  """
  A module is the way we implement a data source for a
  SysSQL table.

  Implementations need to have name and columns fields, and
  at least the generate method.
  """
  name = None
  columns = None

  def fetch(self):
    """
    Returns all rows, as a list, with values
    "safe" for JSON serialization.
    """
    rows = []
    for item in self.generate():
      row = []
      for c in xrange(len(self.columns)):
        v = safe_access(self.access, item, c)
        row.append(v)
      rows.append(row)
    return rows

  def generate(self):
    """Yields datums that are to be accessed with access."""
    pass

  def access(self, datum, column_idx):
    """Default is to treat datum as a tuple/array."""
    return datum[column_idx]

  def set_context(self, context):
    self.context = context

def safe_access(accessor, datum, column):
  """
  Utility method to grab a field, with safe types for serialization.
  """
  try:
    val = accessor(datum, column)
    if not is_valid_type(val):
      val = str(val)
  except:
    LOG.warn("Failed to access %s:%s" % (str(datum), str(column)))
    return None
  return val


def is_valid_type(x):
  """
  Returns true for types that we can return to JSON without worry.
  """
  if x is None:
    return True

  if isinstance(x, (int, long, float, str)):
    return True
